﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Linq;
using HeuristicLab.Analysis.SolutionCaching.PermutationEncoding;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.MainForm;
using HeuristicLab.Problems.TravelingSalesman;

namespace HeuristicLab.Analysis.AlgorithmBehavior.Analyzers.Views {
  [View("PermutationConvexHull View")]
  [Content(typeof(PermutationSolutionCache), false)]
  public class PermutationConvexHullView : ConvexHullView<Permutation, PermutationSolutionInformation> {

    protected override double[] ConvertPermutationToVertex(Permutation p) {
      double[] vertex = p.Select(x => (double)x).ToArray();
      return vertex;
    }

    protected override void testConvexHullButton_Click(object sender, System.EventArgs e) {
      for (int i = 0; i < 20; i++) {
        var sols = Content.GetSolutionsFromGeneration(i);
        //double[][] pop = sols.Select(x => ConvertPermutationToVertex(x)).ToArray();  
        //TODO: simCalculators should be somehow be selected based on problem

        double[][] dm = new double[sols.Count][];
        for (int j = 0; j < sols.Count; j++) {
          dm[j] = new double[sols.Count];
        }

        for (int j = 0; j < sols.Count; j++) {
          for (int k = 0; k < sols.Count; k++) {
            dm[j][k] = TSPSimilarityCalculator.CalculateSimilarity(sols[j], sols[k]);
          }
        }

        double[][] popPoints = DistanceMatrixToPoints.ConvertDistanceMatrixToPoints(DistanceMatrixToPoints.TransformToDistances(dm));

        var convexHull = HyperHull.Calculate(popPoints);
        resultsTextBox.Text += "Nr. of Points in Generation " + i + ": " + convexHull.Count + Environment.NewLine;
        resultsTextBox.Text += "Volume of Convex Hull in Generation " + i + " is: " +
                               ConvexHullMeasures.CalculateVolume(convexHull) + Environment.NewLine;
      }
    }
  }
}
