﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;


namespace HeuristicLab.Analysis.AlgorithmBehavior.Analyzers {
  /// <summary>
  /// Based on the idea from: 
  /// http://stackoverflow.com/questions/4901959/find-if-a-point-is-inside-a-convex-hull-for-a-set-of-points-without-computing-th
  /// </summary>
  public class HyperHull {
    public static List<double[]> Calculate(double[][] inputs) {
      int[] labels = new int[inputs.Count()];
      List<double[]> result = new List<double[]>();

      for (int i = 0; i < inputs.Length; i++) {
        labels[i] = -1;
      }
      labels[0] = 1;

      for (int i = 0; i < labels.Count(); i++) {
        if (i > 0) {
          labels[i - 1] = -1;
          labels[i] = 1;
        }

        KernelSupportVectorMachine machine = new KernelSupportVectorMachine(new LinearKernel(), inputs[0].Length);
        SequentialMinimalOptimization smo = new SequentialMinimalOptimization(machine, inputs, labels);
        smo.Complexity = 1.0;
        smo.Epsilon = 0.0000000000001;
        //smo.Tolerance = 0.0001;

        double error = smo.Run();

        bool isConvexHullPoint = true;
        for (int j = 0; j < inputs.Length; j++) {
          int decision = System.Math.Sign(machine.Compute(inputs[j]));
          if (decision != labels[j]) {
            isConvexHullPoint = false;
            break;
          }
        }
        if (isConvexHullPoint) {
          result.Add(inputs[i]);
        }
      }
      return result;
    }
  }
}
